---
title: "Build Targets"
description: "This document will list all supported targets in the Xamarin.Android build process."
ms.prod: xamarin
ms.assetid: 17DE89FF-F316-4620-B865-EF6E0963A29C
ms.technology: xamarin-android
author: jonpryor
ms.author: jopryo
ms.date: 03/29/2022
---

# Build Targets

The following build targets are defined for Xamarin.Android projects.

## Build

Builds the source code within a project and all dependencies.

This target *does not* create an Android package (`.apk` file).
To create an Android package, use the [SignAndroidPackage](#signandroidpackage)
target, *or* set the
[`$(AndroidBuildApplicationPackage)](~/android/deploy-test/building-apps/build-properties.md#androidbuildapplicationpackage)
property to True when building:

```shell
msbuild /p:AndroidBuildApplicationPackage=True App.sln
```

## BuildAndStartAotProfiling

Builds the app with an  embedded AOT profiler, sets the profiler TCP port to
[`$(AndroidAotProfilerPort)`](~/android/deploy-test/building-apps/build-properties.md#androidaotprofilerport),
and starts the default activity.

The default TCP port is `9999`.

Added in Xamarin.Android 10.2.

## Clean

Removes all files generated by the build process.

## FinishAotProfiling

Must be called *after* the [BuildAndStartAotProfiling](#buildandstartaotprofiling)
target.

Collects the AOT profiler data from the device or emulator through the TCP port
[`$(AndroidAotProfilerPort)`](~/android/deploy-test/building-apps/build-properties.md#androidaotprofilerport)
and writes them to
[`$(AndroidAotCustomProfilePath)`](~/android/deploy-test/building-apps/build-properties.md#androidaotcustomprofilepath).

The default values for port and custom profile are `9999` and `custom.aprof`.

To pass additional options to `aprofutil`, set them in the
[`$(AProfUtilExtraOptions)`](~/android/deploy-test/building-apps/build-properties.md#aprofutilextraoptions)
property.

This is equivalent to:

```shell
aprofutil $(AProfUtilExtraOptions) -s -v -f -p $(AndroidAotProfilerPort) -o "$(AndroidAotCustomProfilePath)"
```

Added in Xamarin.Android 10.2.

## GetAndroidDependencies

Creates the `@(AndroidDependency)` item group, which is used by the
[`InstallAndroidDependencies`](#installandroiddependencies) target to determine
which Android SDK packages to install.

## Install

[Creates, signs](#signandroidpackage), and installs the Android package onto
the default device or virtual device.

The [`$(AdbTarget)`](~/android/deploy-test/building-apps/build-properties.md#adbtarget)
property specifies the Android target device the
Android package may be installed to or removed from.

```bash
# Install package onto emulator via -e
# Use `/Library/Frameworks/Mono.framework/Commands/msbuild` on OS X
MSBuild /t:Install ProjectName.csproj /p:AdbTarget=-e
```

## InstallAndroidDependencies

Calls the [`GetAndroidDependencies`](#getandroiddependencies) target, then installs
the Android SDK packages specified in the `@(AndroidDependency)` item group.

The [`$(AndroidManifestType)`](~/android/deploy-test/building-apps/build-properties.md#androidmanifesttype)
MSBuild property controls which
[Visual Studio SDK Manager repository](~/android/get-started/installation/android-sdk.md?tabs=windows#repository-selection)
is used for package name and package version detection, and URLs to download.

## SignAndroidPackage

Creates and signs the Android package (`.apk`) file.

Use with `/p:Configuration=Release` to generate self-contained "Release" packages.

## StartAndroidActivity

Starts the default activity on the device or the running emulator.

To start a different activity, set the
[`$(AndroidLaunchActivity)`](~/android/deploy-test/building-apps/build-properties.md#androidlaunchactivity)
property to the activity name.

This is equivalent to:

```shell
adb shell am start @PACKAGE_NAME@/$(AndroidLaunchActivity)
```

Added in Xamarin.Android 10.2.

## StopAndroidPackage

Completely stops the application package on the device or the running emulator.

This is equivalent to:

```shell
adb shell am force-stop @PACKAGE_NAME@
```

Added in Xamarin.Android 10.2.

## Uninstall

Uninstalls the Android package from the default device or virtual device.

The [`$(AdbTarget)`](~/android/deploy-test/building-apps/build-properties.md#adbtarget)
property specifies the Android target device the
Android package may be installed to or removed from.

## UpdateAndroidResources

Updates the `Resource.designer.cs` file.

This target is usually called by the IDE when new resources are added to the project.
